@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2017-2019, QIIME 2 development team.
+# Copyright (c) 2017-2020, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -8,6 +8,7 @@
 
 import gzip
 import os
+import shutil
 import subprocess
 import tempfile
 
@@ -83,8 +84,20 @@ def _rename_files(seqs_dir_fmt, per_sample_dir_fmt, barcode_series):
             src = os.path.join(str(per_sample_dir_fmt),
                                '%s.%d.fastq.gz' % (sample_id,
                                                    read_direction))
+
+            # TODO: remove this outer guard when we upgrade to cutadapt 3
             if os.path.isfile(src):
-                os.rename(src, str(out_fp))
+                if out_fp.exists():
+                    _merge_files(src, str(out_fp))
+                    os.remove(src)
+                else:
+                    os.rename(src, str(out_fp))
+
+
+def _merge_files(src, dst):
+    with gzip.open(src, mode='rt', encoding='ascii') as src_fh, \
+            gzip.open(dst, mode='at', encoding='ascii') as dst_fh:
+        shutil.copyfileobj(src_fh, dst_fh)
 
 
 def _write_barcode_fasta(barcode_series, barcode_fasta):
@@ -106,8 +119,8 @@ def _write_empty_fastq_to_mux_barcode_in_seq_fmt(seqs_dir_fmt):
         seqs_dir_fmt.file.write_data(fastq, FastqGzFormat)
 
 
-def _demux(seqs, forward_barcodes, reverse_barcodes, error_tolerance,
-           mux_fmt, batch_size, minimum_length):
+def _demux(seqs, per_sample_sequences, forward_barcodes, reverse_barcodes,
+           error_tolerance, mux_fmt, batch_size, minimum_length):
     fwd_barcode_name = forward_barcodes.name
     forward_barcodes = forward_barcodes.drop_missing_values()
     barcodes = forward_barcodes.to_series().to_frame()
@@ -130,14 +143,16 @@ def _demux(seqs, forward_barcodes, reverse_barcodes, error_tolerance,
 
         if samples_w_missing_barcodes:
             raise ValueError('The following samples do not have both '
-                             'forward and reverse barcodes: %s'
+                             'forward and reverse barcodes (note: if your '
+                             'reads are in single index mixed orientation, '
+                             'try again with all of your barcodes in a single '
+                             'metadata column): %s'
                              % ', '.join(sorted(samples_w_missing_barcodes)))
         if samples_w_dup_barcode_pairs:
             raise ValueError('The following samples have duplicate barcode'
                              ' pairs: %s' %
                              ', '.join(sorted(samples_w_dup_barcode_pairs)))
 
-    per_sample_sequences = CasavaOneEightSingleLanePerSampleDirFmt()
     n_samples = len(barcodes)
     if batch_size > n_samples:
         raise ValueError('The batch_size (%d) cannot be greater than the '
@@ -164,13 +179,13 @@ def _demux(seqs, forward_barcodes, reverse_barcodes, error_tolerance,
         if reverse_barcodes is not None:
             open_fhs['rev'].close()
         previous_untrimmed = current_untrimmed
-        # Only use the forward barcode in the renamed files
-    _rename_files(seqs, per_sample_sequences, barcodes[fwd_barcode_name])
 
+    # Only use the forward barcode in the renamed files
+    _rename_files(seqs, per_sample_sequences, barcodes[fwd_barcode_name])
     muxed = len(list(per_sample_sequences.sequences.iter_views(FastqGzFormat)))
     if muxed == 0:
         raise ValueError('No samples were demultiplexed.')
-    return per_sample_sequences, previous_untrimmed
+    return previous_untrimmed
 
 
 def demux_single(seqs: MultiplexedSingleEndBarcodeInSequenceDirFmt,
@@ -180,9 +195,14 @@ def demux_single(seqs: MultiplexedSingleEndBarcodeInSequenceDirFmt,
                  minimum_length: int = 1) -> \
                     (CasavaOneEightSingleLanePerSampleDirFmt,
                      MultiplexedSingleEndBarcodeInSequenceDirFmt):
+    per_sample_sequences = CasavaOneEightSingleLanePerSampleDirFmt()
     mux_fmt = MultiplexedSingleEndBarcodeInSequenceDirFmt
-    return _demux(seqs, barcodes, None, error_rate, mux_fmt, batch_size,
-                  minimum_length)
+
+    untrimmed = _demux(
+        seqs, per_sample_sequences, barcodes, None, error_rate, mux_fmt,
+        batch_size, minimum_length)
+
+    return per_sample_sequences, untrimmed
 
 
 def demux_paired(seqs: MultiplexedPairedEndBarcodeInSequenceDirFmt,
@@ -190,9 +210,33 @@ def demux_paired(seqs: MultiplexedPairedEndBarcodeInSequenceDirFmt,
                  reverse_barcodes: qiime2.CategoricalMetadataColumn = None,
                  error_rate: float = 0.1,
                  batch_size: int = 0,
-                 minimum_length: int = 1) -> \
+                 minimum_length: int = 1,
+                 mixed_orientation: bool = False) -> \
                     (CasavaOneEightSingleLanePerSampleDirFmt,
                      MultiplexedPairedEndBarcodeInSequenceDirFmt):
+    if mixed_orientation and reverse_barcodes is not None:
+        raise ValueError('Dual-indexed barcodes for mixed orientation '
+                         'reads are not supported.')
+
+    per_sample_sequences = CasavaOneEightSingleLanePerSampleDirFmt()
     mux_fmt = MultiplexedPairedEndBarcodeInSequenceDirFmt
-    return _demux(seqs, forward_barcodes, reverse_barcodes, error_rate,
-                  mux_fmt, batch_size, minimum_length)
+
+    untrimmed = _demux(
+        seqs, per_sample_sequences, forward_barcodes, reverse_barcodes,
+        error_rate, mux_fmt, batch_size, minimum_length)
+
+    if mixed_orientation:
+        fwd = untrimmed.forward_sequences.view(FastqGzFormat)
+        rev = untrimmed.reverse_sequences.view(FastqGzFormat)
+
+        remaining_seqs = MultiplexedPairedEndBarcodeInSequenceDirFmt()
+        # fwd -> rev && rev -> fwd
+        remaining_seqs.forward_sequences.write_data(rev, FastqGzFormat)
+        remaining_seqs.reverse_sequences.write_data(fwd, FastqGzFormat)
+
+        untrimmed = _demux(
+            remaining_seqs, per_sample_sequences, forward_barcodes,
+            reverse_barcodes, error_rate, mux_fmt, batch_size,
+            minimum_length)
+
+    return per_sample_sequences, untrimmed
